#version 100

precision mediump float;

varying vec4 var_position;
varying vec2 var_texcoord0;

varying vec4 srodek_metaball;

uniform sampler2D tex_metaball;
uniform sampler2D tex_base;
uniform sampler2D tex_stars;
uniform sampler2D tex_background;
uniform sampler2D tex_button_shine;
uniform sampler2D tex_gui;



uniform vec4 color;
uniform vec4 obwodka;

varying vec2 vPosition;

#define SPEED (1.0 / 150.0)
#define SMOOTH_DIST 0.15

#define PI 3.14159265359
#define moduloTime 1

float grid(vec2 uv, float grid_scale, float line_thickness) {
	vec2 box_uv = fract(grid_scale * uv);
	vec2 grid_vec = 1.0 - smoothstep(0.0, line_thickness, box_uv);

	return 1.0 - dot(grid_vec, vec2(1.0));
}

/*
* Fitzgibbon's approximation of Barrel/Pincushion distortion. 
* Formula and explanation found at:
* http://marcodiiga.github.io/radial-lens-undistortion-filtering
* 
* r     - distance from the center of distortion
* alpha - distortion parameter. negative for barrel distortion,
* 		   positive for pincushion distortion.
*/
vec2 lens_distortion(vec2 r, float alpha) {
	return r * (1.0 - alpha * dot(r, r));

}

vec2 zoom_point(vec2 uv, vec2 point, float zoom) {
	//translate so the point is at the origin, scale, and translate back to the point
	return (uv - point) / zoom + point;
}

vec4 shine_supernova(vec2 uv_initial) {
	// set up our coordinate system
	vec2 centerOffset = vec2(0.32, -0.09);
	vec2 effectCenterPoint = vec2(0.5, 0.5) + centerOffset;

	if (distance(effectCenterPoint, uv_initial) > 0.175)
		return vec4(0.0,0.0,0.0,0.0);
	
	vec2 res = vec2(1.0, 1.0); // <3>
	vec2 uv = (uv_initial - centerOffset) * res.xy - 0.5;
	float time = obwodka.w; // <4>

	
	
	// get our distance and angle
	float dist = length(uv);
	float angle = (atan(uv.y, uv.x) + PI) / (2.0 * PI);

	// texture lookup
	vec3 textureDist;
	float correctedTime = time;
	correctedTime = mod(correctedTime * SPEED, float(moduloTime));
	 
	textureDist  = texture2D(tex_stars, vec2(correctedTime, angle)).xyz;

	vec3 ichannel = vec3(0,0,0);
	
	vec3 normal = ichannel; //texture(tex_base, uv).xyz;

	// manipulate distance
	textureDist *= 0.12;
	textureDist += 0.05;

	// set RGB
	vec3 color = vec3(0.0,0.0,0.0);

	if (dist < textureDist.x)
	color.x += smoothstep(0.0,SMOOTH_DIST, textureDist.x - dist);

	if (dist < textureDist.y)
	color.y += smoothstep(0.0,SMOOTH_DIST, textureDist.y - dist);

	if (dist < textureDist.z)
	color.z += smoothstep(0.0,SMOOTH_DIST, textureDist.z - dist);


	// set final color
	vec4 fragColor = vec4(color + normal,1.0);
	return fragColor;
}

vec4 square_shine() {
	// Pobierz współrzędne piksela
	float time = obwodka.w;
	// Pobierz współrzędne piksela
	vec2 uv = var_texcoord0.xy;

	// Ustaw kolor piksela na czarny
	vec4 color = vec4(0.0, 0.0, 0.0, 1.0);

	// Oblicz odległość od lewej krawędzi (X = 0) do punktu na przycisku
	float distance_from_left_edge = uv.x;

	// Oblicz maksymalną odległość poziomą, na jaką może się przesunąć promień (sqrt(2) = 1)
	float max_horizontal_distance = 1.0;

	// Oblicz poziomą pozycję promienia
	float horizontal_position = mod(time, max_horizontal_distance);

	// Sprawdź, czy promień jest w fazie "przemieszczania się w prawo" lub "powrotu na lewą stronę"
	bool moving_right = horizontal_position < 0.5 * max_horizontal_distance;

	// Oblicz pionową pozycję promienia (używając współczynnika kierunkowego 1)
	float vertical_position = horizontal_position;

	// Jeśli promień jest w fazie "powrotu na lewą stronę", oblicz jego pionową pozycję na podstawie pozostałej odległości
	if (!moving_right) {
		vertical_position = max_horizontal_distance - horizontal_position;
	}

	// Oblicz odległość od promienia pod kątem 45 stopni
	float distance_from_diagonal = abs(vertical_position - uv.y);

	// Jeśli odległość od promienia pod kątem 45 stopni jest mniejsza niż 0.02 (grubość promienia), ustaw kolor na biały
	if (distance_from_diagonal < 0.02) {
		color = vec4(1.0, 1.0, 1.0, 1.0);
	}

	// Ustaw kolor piksela na podstawie efektu promienia
	gl_FragColor = color;
	return gl_FragColor;
}


void main()
{
	
	vec4 metaball = texture2D(tex_metaball, var_texcoord0.xy);

	
	vec2 res = vec2(1,1);
	vec2 uv0 = var_texcoord0.xy;

	
	//Centered UV Coordinates accounting for aspect ratio.
	vec2 uv = var_texcoord0.xy;

	//Fix thumbnail
	// vec2 mouse_uv = vec2(uv0.x + metaball.r/255, uv0.y + metaball.g/255);
	vec2 mouse_uv = vec2(uv0.x + metaball.r/255, uv0.y + metaball.g/255);
	
	//scale of the un-zoomed grid
	float grid_size = 20.0;

	//Draw the main grid at regular size
	vec4 fragColor = vec4(grid(uv, grid_size,  0.1));

	//Radius from the mouse that represents the size
	//of the magnifying glass
	float lens_radius = 0;

	//Outline of the magnifying glass
	float mouse_dist = distance(uv, mouse_uv);
	if (mouse_dist < lens_radius + 5)
	vec4 fragColor = vec4(0.0);

	//zoom in and distort space around the mouse
	if (metaball.a > 0.85) {
		float zoom = 1.0;

		//Distort space inside the lens with pincushion distortion
		vec2 distortion = lens_distortion((uv - mouse_uv )*2, 0);
		uv0.x = uv0.x + distortion.x;
		uv0.y = uv0.y + distortion.y;

		//Zoom in, keeping the distortion effect
		vec2 zoomed = zoom_point(uv + distortion, mouse_uv, 2 * zoom);

		//Draw the grid with zoom and distortion
		float brightness = grid(zoomed, grid_size, 0.1);
		fragColor = vec4(brightness);
	}

	float threshold = 0.9;
	float obwodka_threshold = 0.98;


	if (metaball.a > obwodka_threshold) {
		int wykladnik = -4;
		vec4 newColor = vec4(obwodka.r * pow(metaball.b, wykladnik), obwodka.g * pow(metaball.b, wykladnik), obwodka.b * pow(metaball.b, wykladnik), color.a);
		metaball = newColor;
	}
	else if (metaball.a > threshold && metaball.a < obwodka_threshold) {
		metaball = color;
	}
	else {
		metaball = vec4(0,0,0,0);
	}
	// metaball = vec4(1,0,0,0);
	vec4 current_pixel = texture2D(tex_background, uv0);

	current_pixel += shine_supernova(uv0);
 
	vec4 baseElementsPixel = texture2D(tex_base, uv0);
	current_pixel += baseElementsPixel;

	vec4 guiElementsPixel = texture2D(tex_gui, uv0);
 
	if (guiElementsPixel.a > 0.3) {
		current_pixel = guiElementsPixel;
		current_pixel += texture2D(tex_button_shine, uv0);
	}

	
	current_pixel += metaball;

	// tylko na telefon zeby bylo cos widac
	current_pixel = texture2D(tex_background, uv0);
	
	
	gl_FragColor = current_pixel;
}




