local mainSavesFilename = "sys_save_load"

goldKeysGlobal = "goldKeys"
rotationPointsGlobal = "rotationPoints"
tacticsPointsGlobal = "tacticsPoints"
lessonPuzzleGlobal = "lessonPuzzle"

local goldKeysCount = "gold_keys_count"
local rotationPointsCount = "rotation_points_count"
local lessonPuzzleCount = "puzzle_elements_count"
local rankedTacticsLeftCount = "ranked_tactics_left_count"
local trainTacticsLeftCount = "train_tactics_left_count"
local areAdsRemoved = "are_ads_removed"

local rankedTacticsScore = "ranked_tactics_score"
local trainTacticsDifficulty = "train_tactics_difficulty"
local trainTacticsNumberOfPuzzles = "train_tactics_number_of_puzzles"

local possibleTrainingDifficulties = {}
possibleTrainingDifficulties.easy = "easy"
possibleTrainingDifficulties.medium = "medium"
possibleTrainingDifficulties.hard = "hard"

local possibleTrainingPuzzlesCount = {3, 4, 5, 6}

local lastTimeOfChestTake = "last_time_of_chest_take"
local lastDayWhenWeGavedRewards = "last_day_when_we_gave_rewards"

local spinCollectedFrames = "spin_collected_frames"
-- local defalutSpinTable = {true, true, true, true, true, true, true, true, true, true}
local defalutSpinTable = {}

local function loadElement(saveLoadFilename, elemName, zeroValue)
	local filename = sys.get_save_file(saveLoadFilename, elemName)
	local data = sys.load(filename)
	return data[elemName] or zeroValue
end

function loadGoldKeysCount()
	return loadElement(mainSavesFilename, goldKeysCount, 0)
end

function loadRotationPointsCount()
	return loadElement(mainSavesFilename, rotationPointsCount, 0)
end

function loadLessonPuzzleCount()
	return loadElement(mainSavesFilename, lessonPuzzleCount, 0)
end

function loadRankedTacticsLeftCount()
	return loadElement(mainSavesFilename, rankedTacticsLeftCount, 10)
end

function loadTrainTacticsLeftCount()
	return loadElement(mainSavesFilename, trainTacticsLeftCount, 10)
end

function loadAreAdsRemovedLeftCount()
	return loadElement(mainSavesFilename, areAdsRemoved, false)
end

function loadRankedTacticsScore()
	return loadElement(mainSavesFilename, rankedTacticsScore, 0)
end

function loadTrainTacticsDifficulty()
	return loadElement(mainSavesFilename, trainTacticsDifficulty, possibleTrainingDifficulties.easy)
end

function loadTrainTacticsNumberOfPuzzles()
	return loadElement(mainSavesFilename, trainTacticsNumberOfPuzzles, 0)
end

function loadLastTimeOfChestTake()
	return loadElement(mainSavesFilename, lastTimeOfChestTake, os.date("%Y-%m-%d",os.time()))
end

function loadLastDayWhenWeGaveRewards()
	return loadElement(mainSavesFilename, lastDayWhenWeGavedRewards, os.date("%Y-%m-%d",os.time()-24*60*60))
end

function loadSpinCollectedTableFrames()
	return loadElement(mainSavesFilename, spinCollectedFrames, defalutSpinTable)
end

function loadAllData(dataTable)
	dataTable.goldKeysCount = loadGoldKeysCount()
	dataTable.rotationPointsCount = loadRotationPointsCount()
	dataTable.lessonPuzzleCount = loadLessonPuzzleCount()
	dataTable.rankedTacticsLeftCount = loadRankedTacticsLeftCount()
	dataTable.trainTacticsLeftCount = loadTrainTacticsLeftCount()
	dataTable.areAdsRemoved = loadAreAdsRemovedLeftCount()
	dataTable.rankedTacticsScore = loadRankedTacticsScore()
	dataTable.trainTacticsDifficulty = loadTrainTacticsDifficulty()
	dataTable.trainTacticsNumberOfPuzzles = loadTrainTacticsNumberOfPuzzles()
	dataTable.lastTimeOfChestTake = loadLastTimeOfChestTake()
	dataTable.lastDayWhenWeGavedRewards = loadLastDayWhenWeGaveRewards()
	dataTable.spinCollectedFrames = loadSpinCollectedTableFrames()
end



function saveSomeResource(typeOfResource, newNumber)
	if typeOfResource == goldKeysGlobal then
		saveGoldKeysCount(newNumber)
	end
	if rotationPointsGlobal == rotationPointsGlobal then
		saveRotationCount(newNumber)
	end
	if tacticsPointsGlobal == tacticsPointsGlobal then
		saveRankedTacticsLeftCount(newNumber)
	end
	if lessonPuzzleGlobal == lessonPuzzleGlobal then
		saveLessonPuzzleCount(newNumber)
	end
end

function zeroSavedData()
	saveGoldKeysCount(0)
	saveRotationCount(0)
	saveLessonPuzzleCount(0)
	saveRankedTacticsLeftCount(0)
	saveTrainTacticsLeftCount(0)
	saveAreAdsRemoved(0)
	saveRankedTacticsScore(0)
	saveTrainTacticsDifficulty(0)
	saveTrainTacticsNumberOfPuzzles(0)
	saveLastTimeOfChestTake(0)
	saveLastDayWhenWeGavedRewards(0)
	saveSpinCollectedRewards(0)
end

function saveGoldKeysCount(currentGoldKeysCount)
	local filename = sys.get_save_file(mainSavesFilename, goldKeysCount)
	sys.save(filename, { gold_keys_count = currentGoldKeysCount })
end

function saveRotationCount(currentRotationCount)
	local filename = sys.get_save_file(mainSavesFilename, rotationPointsCount)
	sys.save(filename, { rotation_points_count = currentRotationCount })
end

function saveLessonPuzzleCount(currentLessonPuzzleCount)
	local filename = sys.get_save_file(mainSavesFilename, lessonPuzzleCount)
	sys.save(filename, { puzzle_elements_count = currentLessonPuzzleCount })
end

function saveRankedTacticsLeftCount(currentRankedTacticsLeftCount)
	local filename = sys.get_save_file(mainSavesFilename, rankedTacticsLeftCount)
	sys.save(filename, { ranked_tactics_left_count = currentRankedTacticsLeftCount })
end

function saveTrainTacticsLeftCount(currentTrainTacticsLeftCount)
	local filename = sys.get_save_file(mainSavesFilename, trainTacticsLeftCount)
	sys.save(filename, { train_tactics_left_count = currentTrainTacticsLeftCount })
end

function saveAreAdsRemoved(currentAreAdsRemoved)
	local filename = sys.get_save_file(mainSavesFilename, areAdsRemoved)
	sys.save(filename, { are_ads_removed = currentAreAdsRemoved })
end

function saveRankedTacticsScore(currentRankedTacticsScore)
	local filename = sys.get_save_file(mainSavesFilename, rankedTacticsScore)
	sys.save(filename, { ranked_tactics_score = currentRankedTacticsScore })
end

function saveTrainTacticsDifficulty(currentTrainTacticsDifficulty)
	local filename = sys.get_save_file(mainSavesFilename, trainTacticsDifficulty)
	sys.save(filename, { train_tactics_difficulty = currentTrainTacticsDifficulty })
end

function saveTrainTacticsNumberOfPuzzles(currentTrainTacticsNumberOfPuzzles)
	local filename = sys.get_save_file(mainSavesFilename, trainTacticsNumberOfPuzzles)
	sys.save(filename, { train_tactics_number_of_puzzles = currentTrainTacticsNumberOfPuzzles })
end

function saveLastTimeOfChestTake(currentLastTimeOfChestTaken)
	local filename = sys.get_save_file(mainSavesFilename, lastTimeOfChestTake)
	sys.save(filename, { last_time_of_chest_take = currentLastTimeOfChestTaken })
end

function saveLastDayWhenWeGavedRewards(currentLastDayWhenWeGavedRewards)
	local filename = sys.get_save_file(mainSavesFilename, lastDayWhenWeGavedRewards)
	sys.save(filename, { last_day_when_we_gave_rewards = currentLastDayWhenWeGavedRewards })
end

function saveSpinCollectedRewards(currentSpinCollectedFrames)
	local filename = sys.get_save_file(mainSavesFilename, spinCollectedFrames)
	sys.save(filename, { spin_collected_frames = currentSpinCollectedFrames })
end